# -*- coding: utf-8 -*-
from django.utils.translation import gettext_lazy as _
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.response import Response

from thx.api.mixins import ThuxModelMixin
from thx.role.core.views import ThuxRoleViewSetMixin

from ... import models
from . import serializers, filters, permissions

STATUS_CODES = ((0, _('disable')), (1, _('enable')),)


# GENERIC TEMPLATE
class GenericTemplateViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    GenericTemplateViewSet
    -----------

    :action list: Return the generic template list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given generic template.
        `Profiles`: `superuser`, `staff`

    :action light: Return the generic template list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new generic template instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a generic template instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a generic template instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a generic template instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.GenericTemplate.objects.all()
    serializer_class = serializers.GenericTemplateSerializer
    filter_backends = (filters.GenericTemplateFilterBackend,)
    permission_classes = (permissions.GenericTemplatePermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a generic template instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a generic template instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# EMAIL TEMPLATE
class EmailTemplateViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    EmailTemplateViewSet
    -----------

    :action list: Return the email template list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given email template.
        `Profiles`: `superuser`, `staff`

    :action light: Return the email template list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new email template instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a email template instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a email template instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a email template instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.EmailTemplate.objects.all()
    serializer_class = serializers.EmailTemplateSerializer
    filter_backends = (filters.EmailTemplateFilterBackend,)
    permission_classes = (permissions.EmailTemplatePermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a email template instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a email template instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)
